// 2009 Hiroyuki Ogasawara
// vim:ts=4 sw=4:


#include	<stdio.h>
#include	<windows.h>


#define	__USE_DEBUG__	0


namespace RunVimLocal {
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------


struct FileInfo {
	const char*	Fullpath;
	const char*	Filename;
	int		Found;
};


static const char*
getFilename( const char* filename )
{
	const char*	ptr= filename;
	const char*	name= filename;
	int	kanji= FALSE;
	for(; *ptr ; ptr++ ){
		if( kanji ){
			kanji= FALSE;
		}else{
			if( (*ptr >= 0x80 && *ptr < 0xa0) || *ptr >= 0xe0 ){
				kanji= TRUE;
			}else{
				if( *ptr == '/' || *ptr == '\\' || *ptr == ':' ){
					name= ptr+1;
				}
			}
		}
	}
	return	name;
}



static char*
getVimPath( char* str, size_t buflength, const char* title )
{
	const char*	ptr= title;
	for(; *ptr && *ptr != '(' ; ptr++ );
	if( *ptr == '(' ){
		char*	endptr= str + buflength -1;
		for( ptr++ ; str < endptr && *ptr && *ptr != ')' ;){
			*str++= *ptr++;
		}
	}
	*str= '\0';
	return	str;
}


static char*
getVimName( char* str, size_t buflength,
					const char* title )
{
	const char*	ptr= title;
	for(; *ptr == ' ' ; ptr++ );
	for(; *ptr != ' ' ; ptr++ ){
		*str++= *ptr;
	}
	*str= '\0';
	return	str;
}



static BOOL CALLBACK
listFunc( HWND hwnd, LPARAM param )
{
	FileInfo*	finfo= reinterpret_cast<FileInfo*>( param );
	const int	MAX_BUFLENGTH= 1024;
	char	title[MAX_BUFLENGTH];
	if( !RealGetWindowClass( hwnd, title, MAX_BUFLENGTH ) ){
		return	TRUE;
	}
	if( strcmp( title, "Vim" ) ){
		return	TRUE;
	}
	if( !GetWindowText( hwnd, title, MAX_BUFLENGTH ) ){
		return	TRUE;
	}

	const int	MAX_NAMELENGTH= 256;
	const int	MAX_PATHLENGTH= 1024;
	char	path[MAX_PATHLENGTH];
	char*	str= getVimPath( path, MAX_PATHLENGTH-MAX_NAMELENGTH, title );

#if __USE_DEBUG__
	printf( "get path=%s\n", path );
#endif

	*str++= '\\';
	getVimName( str, MAX_NAMELENGTH, title );

#if __USE_DEBUG__
	printf( "get full=%s\n", path );
#endif

	if( !_stricmp( path, finfo->Fullpath ) ){
		SetForegroundWindow( hwnd );
		finfo->Found= TRUE;
		return	FALSE;
	}
	return	TRUE;
}


static void
usage()
{
	puts(	"runvim v1.00 2009 Hiroyuki Ogasawra\n"
			"usage: runvim [options] <filename>\n"
			"  -e<cmdexe>  exename (ex. -ec:/app/vim72/gvim.exe)\n"
			"  -o<cmdopt>  options"
		);
}


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
}


int
main( int argc, char** argv )
{
	const char*	exename= "c:\\app\\vim\\vim72\\gvim.exe";
	const char*	optname= "";
	const char*	filename= "";
	int	newflag= FALSE;
	for(; --argc ;){
		if( **++argv == '-' ){
			switch( (*argv)[1] ){
			case 'e':
				exename= *argv + 2;
				break;
			case 'o':
				optname= *argv + 2;
				break;
			case 'n':
				newflag= TRUE;
				break;
			default:
				RunVimLocal::usage();
				return	0;
			}
		}else{
			filename= *argv;
		}
	}
	RunVimLocal::FileInfo	param;
	param.Found= FALSE;
	if( !newflag && filename && *filename ){
		const int	MAX_BUFLENGTH= 1024;
		char	fullbuf[MAX_BUFLENGTH];
		GetFullPathName( filename, MAX_BUFLENGTH, fullbuf, NULL );
		param.Fullpath= fullbuf;
		param.Filename= RunVimLocal::getFilename( fullbuf );
		EnumWindows( RunVimLocal::listFunc, reinterpret_cast<LPARAM>( &param ) );
	}
	if( !param.Found ){
		STARTUPINFO	startup;
		memset( &startup, 0, sizeof(STARTUPINFO) );
		startup.cb= sizeof(STARTUPINFO);
		PROCESS_INFORMATION	proc;
		memset( &proc, 0, sizeof(PROCESS_INFORMATION) );
		const int	MAX_CMDLINE= 1024;
		char	cmdline[MAX_CMDLINE];
		strcpy_s( cmdline, MAX_CMDLINE, optname );
		strcat_s( cmdline, MAX_CMDLINE, " " );
		strcat_s( cmdline, MAX_CMDLINE, filename );
		CreateProcess(
			exename,
			cmdline,
			NULL,
			NULL,
			FALSE,
			0, 	// flags
			NULL, // env
			NULL, // current dir
			&startup, // startup
			&proc // proc
				);
#if 0
		printf( "proc=%p thread=%p pid=%d tid=%d\n",
			proc.hProcess,
			proc.hThread,
			proc.dwProcessId,
			proc.dwThreadId );
#endif
	}
	return	0;
}

